@@ -23,7 +23,7 @@ class RegisterSaveAreaSimplifier(OptimizationPass):
     Optimizes away register spilling effects, including callee-saved registers.
     """
 
-    ARCHES = ["X86", "AMD64", "ARM", 'ARMEL', 'ARMHF', "ARMCortexM", ]
+    ARCHES = ["X86", "AMD64", "ARM", 'ARMEL', 'ARMHF', "ARMCortexM", "MIPS32", "MIPS64", ]
     PLATFORMS = ["cgc", "linux"]
     STAGE = OptimizationPassStage.AFTER_GLOBAL_SIMPLIFICATION
     NAME = "Simplify register save areas"
