@@ -18,7 +18,7 @@ class ConstantDereferences(PeepholeOptimizationExprBase):
         if isinstance(expr.addr, Const):
             # is it loading from a read-only section?
             sec = self.project.loader.find_section_containing(expr.addr.value)
-            if sec is not None and sec.is_readable and not sec.is_writable:
+            if sec is not None and sec.is_readable and (not sec.is_writable or "got" in sec.name):
                 # do we know the value that it's reading?
                 try:
                     val = self.project.loader.memory.unpack_word(expr.addr.value, size=self.project.arch.bytes)
