@@ -15,5 +15,6 @@ class fileno(angr.SimProcedure):
         io_file_data = io_file_data_for_arch(self.state.arch)
 
         # Get the file descriptor from FILE struct
-        result = self.state.mem[f + io_file_data['fd']].int.resolved
-        return result.sign_extend(self.arch.bits - len(result))
+        return self.state.mem[f + io_file_data['fd']].int.resolved
+
+fileno_unlocked = fileno
